<?php
namespace WPUmbrella\Actions\Queue\Backup;

use WPUmbrella\Core\Constants\CodeResponse;
use WPUmbrella\Core\Hooks\ExecuteHooks;
use WPUmbrella\Core\Backup\Builder\BackupBuilder;
use WPUmbrella\Services\Backup\V2\BackupManageProcess;
use WPUmbrella\Services\Backup\QueueRunner\V2\BackupQueueRunnerFiles;


class BackupQueueV2 implements ExecuteHooks
{
    public function hooks()
    {
        add_action(BackupManageProcess::ACTION_BACKUP_FILES, [$this, 'runFiles']);
        add_action(BackupManageProcess::ACTION_BACKUP_DATABASE, [$this, 'runDatabase']);
        add_action(BackupManageProcess::ACTION_BACKUP_PREPARE_BATCH_DATABASE, [$this, 'runPrepareBatchDatabase']);
        add_action(BackupManageProcess::ACTION_BACKUP_CHECK_BATCH_DATABASE, [$this, 'runCheckBatchDatabase']);
        add_action(BackupManageProcess::ACTION_BACKUP_CLEANUP, [$this, 'cleanup']);
		add_action('action_scheduler_failed_action', [$this, 'cleanupBackupIfFailed']);
        add_action('action_scheduler_failed_execution', [$this, 'cleanupBackupIfFailed']);
    }

    public function cleanupBackupIfFailed($actionId)
    {
        try {
            $action = \ActionScheduler::store()->fetch_action($actionId);
            $hook = $action->get_hook();

			$manageProcess = wp_umbrella_get_service('BackupManageProcess');


            switch ($hook) {
				case BackupManageProcess::ACTION_BACKUP_FILES:
					$data = $manageProcess->getBackupData();

					if($data === null){
						return;
					}

					$data = $data->getData();
					$data['type'] = 'file';
					wp_umbrella_get_service('BackupApi')->postErrorBackup($data);
					wp_umbrella_get_service('BackupManageProcess')->actionSchedulerCleanup();
					break;
				case BackupManageProcess::ACTION_BACKUP_DATABASE:
				case BackupManageProcess::ACTION_BACKUP_PREPARE_BATCH_DATABASE:
				case BackupManageProcess::ACTION_BACKUP_CHECK_BATCH_DATABASE:
					$data = $manageProcess->getBackupData();

					if($data === null){
						return;
					}

					$data = $data->getData();
					$data['type'] = 'database';
					wp_umbrella_get_service('BackupApi')->postErrorBackup($data);
					wp_umbrella_get_service('BackupManageProcess')->actionSchedulerCleanup();
                    break;
            }

        } catch (\Exception $e) {
			// Do nothing
        }
    }

    public function runFiles()
    {

		if(!defined('WP_UMBRELLA_INIT_BACKUP')){
			define('WP_UMBRELLA_INIT_BACKUP', true);
		}

        $runner = wp_umbrella_get_service('BackupQueueRunnerFilesV2');
		$runner->setApi(wp_umbrella_get_service('BackupApi'));
		$response = $runner->run();

		$code = $response['data']['code'];

		switch($code){
			case CodeResponse::BACKUP_NEXT_PART_FILES:
				wp_umbrella_get_service('BackupManageProcess')->actionSchedulerBatchFiles();
				break;
			case CodeResponse::BACKUP_FILES_FINISH:
				$manageProcess = wp_umbrella_get_service('BackupManageProcess');
				$data = $manageProcess->getBackupData();
				if($data->getIsDatabaseSourceRequired()){
					$manageProcess->actionSchedulerBatchDatabase();
				}
				else{
					// finish
					$manageProcess->actionSchedulerCleanup();
				}
				break;
		}
    }

    public function runDatabase()
    {
		if(!defined('WP_UMBRELLA_INIT_BACKUP')){
			define('WP_UMBRELLA_INIT_BACKUP', true);
		}

		$runner = wp_umbrella_get_service('BackupQueueRunnerDatabaseV2');
		$runner->setApi(wp_umbrella_get_service('BackupApi'));
		$response = $runner->run();

		$code = $response['data']['code'];

		switch($code){
			case CodeResponse::BACKUP_TABLE_CHECK_BATCH:
				wp_umbrella_get_service('BackupManageProcess')->actionSchedulerCheckBatchDatabase();
				break;
			case CodeResponse::BACKUP_NEXT_PART_DATABASE:
				wp_umbrella_get_service('BackupManageProcess')->actionSchedulerBatchDatabase();
				break;
			case CodeResponse::BACKUP_TABLE_NEED_BATCH:
				wp_umbrella_get_service('BackupManageProcess')->actionSchedulerPrepareBatchDatabase();
				break;
			case CodeResponse::BACKUP_DATABASE_FINISH:
				wp_umbrella_get_service('BackupManageProcess')->actionSchedulerCleanup();
				break;
		}

    }

	public function runPrepareBatchDatabase()
    {
		if(!defined('WP_UMBRELLA_INIT_BACKUP')){
			define('WP_UMBRELLA_INIT_BACKUP', true);
		}

		$runner = wp_umbrella_get_service('BackupQueuePrepareDatabaseBatch');
		$runner->setTypeClient("wordpress");
		$response = $runner->run();

		$success = $response['success'];

		if($success){
			wp_umbrella_get_service('BackupManageProcess')->actionSchedulerBatchDatabase();
		}
		else{
			// Error
			$manageProcess = wp_umbrella_get_service('BackupManageProcess');
			$data = $manageProcess->getBackupData();
			$data = $data->getData();
			$data['type'] = 'database';
			wp_umbrella_get_service('BackupApi')->postErrorBackup($data);
			wp_umbrella_get_service('BackupManageProcess')->actionSchedulerCleanup();
		}
    }

	/**
	 * Check batch database
	 */
	public function runCheckBatchDatabase()
    {
		if(!defined('WP_UMBRELLA_INIT_BACKUP')){
			define('WP_UMBRELLA_INIT_BACKUP', true);
		}

		$runner = wp_umbrella_get_service('BackupQueueCheckDatabaseBatch');
		$runner->setTypeClient("wordpress");
		$response = $runner->run([
			'divided_memory' => 2
		]);

		$success = $response['success'];

		if($success){
			wp_umbrella_get_service('BackupManageProcess')->actionSchedulerBatchDatabase();
		}
		else{
			// Error
			$manageProcess = wp_umbrella_get_service('BackupManageProcess');
			$data = $manageProcess->getBackupData();
			$data = $data->getData();
			$data['type'] = 'database';
			wp_umbrella_get_service('BackupApi')->postErrorBackup($data);
			wp_umbrella_get_service('BackupManageProcess')->actionSchedulerCleanup();
		}
    }

	public function cleanup(){
		wp_umbrella_get_service('BackupManageProcess')->deleteProcess();
		wp_umbrella_get_service('BackupExecutorV2')->cleanup();
		wp_umbrella_get_service('BackupStandalone')->deleteKernel();
	}
}
