<?php
namespace WPUmbrella\Services\Backup\V2;


use WPUmbrella\Models\Backup\BackupProcessedData;
use WPUmbrella\Core\Exceptions\BackupNotCreated;
use WPUmbrella\Services\Backup\BackupBatchData;
use WPUmbrella\Models\Backup\V2\BackupConfigData;
use WPUmbrella\Core\Constants\BackupData;


class BackupManageProcess
{
    const GROUP = 'wp-umbrella-v2';

    const ACTION_BACKUP_FILES = 'wp_umbrella_backup_files_v2_batch';

    const ACTION_BACKUP_DATABASE = 'wp_umbrella_backup_database_v2_batch';

	const ACTION_BACKUP_PREPARE_BATCH_DATABASE = 'wp_umbrella_backup_prepare_batch_database';

	const ACTION_BACKUP_CHECK_BATCH_DATABASE = 'wp_umbrella_backup_check_batch_database';

	const ACTION_BACKUP_CLEANUP = 'wp_umbrella_backup_cleanup';

	const FILE_CONFIG = 'config.php';

	protected $data;

	public function getPathFileConfig(){
		return sprintf('%s/%s', WP_UMBRELLA_DIR_WPU_BACKUP, self::FILE_CONFIG);
	}


	/**
	 * @param array $params
	 * 	- string $params['title']
	 * 	- string $params['suffix']
	 * 	- string $params['date']
	 * 	- string $params['database']
	 * 	- string $params['part']
	 * 	- string $params['backupId']
	 *
	 * @return string
	 */
	public function createDefaultName($params)
    {
		$title = $params['title'] ?? '';
		$suffix = $params['suffix'] ?? '';
		$database = $params['database'] ?? false;
		$part = $params['part'] ?? null;
		$backupId = $params['backupId'] ?? null;

        $name = 'backup-';
        if ($database) {
            $name .= 'database-';
        }

        $value = $params['date'] ?? date('Y-m-d-H');
        if ($backupId !== null) {
            $value = $backupId;
        }

        $basename = sprintf('%s-%s', substr($title, 0, 5), $value);
        $name .= $basename;

        if (!empty($suffix)) {
            $name .= '-' . $suffix;
        }

        if ($part !== null && !$database) {
            $name .= '-part-' . $part;
        }

        return strtolower($name);
    }


	/**
	 * Init config data for backup
	 * @param array $params
	 */
	protected function initData($params){

		if(isset($params['max_file_mo_size'])){
			$moSize = $params['max_file_mo_size'];
		}
		else{
			$moSize = wp_umbrella_get_service('BackupFinderConfiguration')->getMaxMoBatchSize();
		}
		$size = sprintf('<= %sM', $moSize);

		$maxSizeInBytes = wp_umbrella_get_service('BackupFinderConfiguration')->getMaxMoInBytesBatchSize();

        $host = DB_HOST;
        if (apply_filters('wp_umbrella_explode_host', true) && strpos($host, 'localhost') !== false && strpos($host, ':') !== false) {
            $host = explode(':', $host)[0];
        }

		$suffix = isset($params['suffix']) ? $params['suffix'] : '';
		$isScheduled = isset($params['is_scheduled']) ? $params['is_scheduled'] : '1';

		$title = sanitize_title(get_bloginfo('name'));
		$date = date('Y-m-d-H');

		$nameBackupDatabase = $this->createDefaultName([
			'title' => $title,
			'suffix' => $suffix,
			'date' => $date,
			'database' => true
		]);

		$nameBackupFile = $this->createDefaultName([
			'title' => $title,
			'date' => $date,
			'suffix' => $suffix,
		]);

		$excludeTables = isset($params['exclude_tables']) ? $params['exclude_tables'] : [];

		$highTables = wp_umbrella_get_service('BackupDatabaseConfigurationV2')->getHighDataTables();

		$tables = wp_umbrella_get_service('BackupDatabaseConfigurationV2')->getTablesConfiguration($excludeTables,[
			'memory_limit_divided_by' => isset($params['memory_limit_divided_by']) ? $params['memory_limit_divided_by'] : BackupData::DEFAULT_MEMORY_LIMIT_DIVIDED_BY,
			'high_tables' => $highTables
		]);

		$memoryLimit = wp_umbrella_get_service('WordPressProvider')->getMemoryLimitBytes();
		$dividedBy = $options['memory_limit_divided_by'] ?? BackupData::DEFAULT_MEMORY_LIMIT_DIVIDED_BY;
		$maximumLinesByBatch = $options['maximum_lines_by_batch'] ?? BackupData::MAXIMUM_LINES_BY_BATCH;

		$data = [
			'title' => $title,
			'date' => $date,
			'suffix' => $suffix,
			'is_scheduled' => $isScheduled,
			'backupId' => isset($params['backupId']) ? $params['backupId'] : null,
            'incremental_date' => isset($params['incremental_date']) ? $params['incremental_date'] : null,
			'timestamp_start_date' => time(),
			'file' => [
				'finish' => false,
				'name' => $nameBackupFile,
				'mode' => 'normal',
				'required' =>  isset($params['files']) ? $params['files'] : true,
				'base_directory' => isset($params['base_directory']) ? $params['base_directory'] : ABSPATH,
				'exclude' => isset($params['exclude_files']) ? $params['exclude_files'] : [],
				'zips_sent' => [],
				'batch' => [
					'iterator_position' => 0, // Offset
					'max_size' => $maxSizeInBytes,
					'part' => 1,
					'size' => $size,
				],
			],
			'database' => [
				'finish' => false,
				'name' => $nameBackupDatabase,
				'required' => isset($params['sql']) ? $params['sql'] : true,
				'exclude' => $excludeTables,
				'zips_sent' => [],
				'tables' => $tables,
				'high_tables' => $highTables,
				'connection' => [
					'database' => DB_NAME,
					'user' => DB_USER,
					'password' => DB_PASSWORD,
					'host' => $host
				],
				'batch' => [
					'memory_limit_divided_by' => $dividedBy,
					'maximum_lines_by_batch' => $maximumLinesByBatch,
					'maximum_memory_authorized' => $memoryLimit / $dividedBy,
					'iterator_position' => 0, // Offset for table
					'part' => 0, // Part for table_batchs
				],
				'table_batchs' => []
			],
			'snapshot' => [
				'plugins'=>  array_map(function($item){
					return $item->getPropertiesValues();
				}, wp_umbrella_get_service('PluginsProvider')->getPlugins([
					'light' => true
				])),
				'theme' => wp_umbrella_get_service('ThemesProvider')->getCurrentTheme(),
				'count_public_posts' => wp_umbrella_get_service('WordPressDataProvider')->countPosts(),
				'count_attachments' => wp_umbrella_get_service('WordPressDataProvider')->countAttachments()
			],
			'wordpress' => wp_umbrella_get_service('WordPressProvider')->getStateWordPress()
		];

		return apply_filters('wp_umbrella_get_data_init_backup', $data);
	}

	public function isWritableData(){
		try {
			return is_writable(dirname($this->getPathFileConfig()));
		} catch (\Exception $e) {
			return true;
		}
	}

    public function isBackupInProgress() {
		if($this->isWritableData()){
			return file_exists( $this->getPathFileConfig() );
		}

		$data = get_option('wp_umbrella_backup_data_process');
		if(!$data){
			return false;
		}

		return true;
    }

    public function deleteProcess(){
        if(! file_exists( $this->getPathFileConfig() ) ){
            return;
        }

        unlink( $this->getPathFileConfig() );
    }


    public function init($params, $version = 'v1')
    {
		if( $this->isBackupInProgress() ){
			throw new BackupNotCreated('Backup in progress');
		}

		$data = $this->initData($params);

		$backupId = isset($params['backupId']) ? $params['backupId'] : null;

		if($backupId === null){
			$backupData = wp_umbrella_get_service('BackupApi')->postInitBackup($data);

			if ($backupData === null || !isset($backupData['id'])) {
				throw new BackupNotCreated('Error Processing Create Backup');
			}

			$data['backupId'] = $backupData['id'];
		}

		// Clean old process if necesary
		wp_umbrella_get_service('BackupExecutorV2')->cleanup();

		// Write file config with data with var_export
		$this->updateBackupData($data);

		// With Action Scheduler
		if($version === 'v1'){
			if($data['file']['required']){
				$this->actionSchedulerBatchFiles();
			}
			else if($data['database']['required']){
				$this->actionSchedulerBatchDatabase();
			}
		}
		else if( $version === 'v2' ){
			wp_umbrella_get_service('BackupStandalone')->writeKernel();
		}

        return $data;
    }

    public function getBackupData(){
		if($this->data !== null){
			return $this->data;
		}

		if(!$this->isWritableData() && function_exists('get_option')){
            $data = get_option('wp_umbrella_backup_data_process');
			if(!$data){
				return null;
			}
			$data = maybe_unserialize($data);
        }
		else{
			$data = include $this->getPathFileConfig();
		}

		$this->data = new BackupConfigData($data);

		return $this->data;
    }

	public function updateBackupData($data){
		$this->data = $data;

		if(! $this->isWritableData()  && function_exists('update_option') ){
			update_option('wp_umbrella_backup_data_process', $data, false);
		}
		else{
			$content = sprintf("<?php

			if(!defined('WP_UMBRELLA_INIT_BACKUP')){
				die('Oups');
			}

			return %s;", var_export($data, true));

			$fp = fopen($this->getPathFileConfig(), 'w');
			if($fp === false){
				$this->actionSchedulerCleanup();
			}
			else{
				fwrite($fp, $content);
				fclose($fp);
			}
		}

	}


    public function actionSchedulerBatchFiles()
    {
        as_schedule_single_action(time() + 1, self::ACTION_BACKUP_FILES, [], self::GROUP);
    }

    public function actionSchedulerBatchDatabase()
    {
        as_schedule_single_action(time() + 1, self::ACTION_BACKUP_DATABASE, [], self::GROUP);
    }

    public function actionSchedulerPrepareBatchDatabase()
    {
        as_schedule_single_action(time() + 1, self::ACTION_BACKUP_PREPARE_BATCH_DATABASE, [], self::GROUP);
    }

	public function actionSchedulerCleanup()
    {
        as_schedule_single_action(time() + 1, self::ACTION_BACKUP_CLEANUP, [], self::GROUP);
    }

	public function actionSchedulerCheckBatchDatabase()
    {
        as_schedule_single_action(time() + 1, self::ACTION_BACKUP_CHECK_BATCH_DATABASE, [], self::GROUP);
    }

    public function unscheduledBatch()
    {
        as_unschedule_action(self::ACTION_BACKUP_FILES, [], self::GROUP);
        as_unschedule_action(self::ACTION_BACKUP_DATABASE, [], self::GROUP);
        as_unschedule_action(self::ACTION_BACKUP_PREPARE_BATCH_DATABASE, [], self::GROUP);
		if(function_exists('delete_option')){
			delete_option('wp_umbrella_backup_data_process');
		}

		$this->deleteProcess();
		wp_umbrella_get_service('BackupExecutorV2')->cleanup();
		wp_umbrella_get_service('BackupStandalone')->deleteKernel();

    }

}
